// OpenLP Mobile Remote
// ---------------------------------------------------------------------------
// Copyright (c) 2008-2019 OpenLP Developers
// ---------------------------------------------------------------------------
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.

import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:url_launcher/url_launcher.dart';

import '../bloc/settings_bloc.dart';
import '../configurations/app_localizations.dart';

class Settings extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    final SettingsBloc bloc = BlocProvider.of<SettingsBloc>(context);
    return Scaffold(
      appBar: AppBar(
        title: Text(AppLocalizations.of(context).translate('settings')),
      ),
      body: BlocBuilder<SettingsBloc, SettingsState>(
        builder: (context, state) => ListView(
          children: <Widget>[
            ListTile(
              title: Text(
                  AppLocalizations.of(context).translate('settings_server_ip')),
              subtitle: Text(state.serverIp ?? ''),
              onTap: () async {
                String title = AppLocalizations.of(context)
                    .translate('dialog_server_ip_title');
                String serverIp = await showDialog<String>(
                  context: context,
                  builder: (context) {
                    return _InputDialog(title,
                        initialValue: state.serverIp,
                        keyboardType: TextInputType.url);
                  },
                );
                bloc.setServerIp(serverIp);
              },
            ),
            ListTile(
              title: Text(AppLocalizations.of(context)
                  .translate('settings_server_port')),
              subtitle: Text('${state.serverPort}'),
              onTap: () async {
                String title = AppLocalizations.of(context)
                    .translate('dialog_server_port_title');
                String serverPortStr = await showDialog<String>(
                  context: context,
                  builder: (context) {
                    return _InputDialog(title,
                        initialValue: '${state.serverPort}',
                        keyboardType: TextInputType.numberWithOptions(
                            decimal: true, signed: true));
                  },
                );
                bloc.setServerPort(serverPortStr);
              },
            ),
            CheckboxListTile(
              title: Text(
                  AppLocalizations.of(context).translate('settings_use_https')),
              value: state.useHttps,
              onChanged: (value) {
                bloc.setUseHttps(value);
              },
            ),
            Divider(),
            CheckboxListTile(
              title: Text(AppLocalizations.of(context)
                  .translate('settings_needs_auth')),
              value: state.needsAuth,
              onChanged: (value) {
                bloc.setNeedsAuth(value);
              },
            ),
            ListTile(
              enabled: state.needsAuth,
              title: Text(
                  AppLocalizations.of(context).translate('settings_user_id')),
              subtitle: Text(state.userId ?? ''),
              onTap: () async {
                String title = AppLocalizations.of(context)
                    .translate('dialog_server_user_id_title');
                String userId = await showDialog<String>(
                  context: context,
                  builder: (context) =>
                      _InputDialog(title, initialValue: state.userId),
                );
                bloc.setUserId(userId);
              },
            ),
            ListTile(
              enabled: state.needsAuth,
              title: Text(
                  AppLocalizations.of(context).translate('settings_user_pass')),
              subtitle: Text(state.userPassword ?? ''),
              onTap: () async {
                String title = AppLocalizations.of(context)
                    .translate('dialog_server_user_pass_title');
                String userPassword = await showDialog<String>(
                  context: context,
                  builder: (context) =>
                      _InputDialog(title, initialValue: state.userPassword),
                );
                bloc.setUserPassword(userPassword);
              },
            ),
            Divider(),
            ListTile(
              title: Text(AppLocalizations.of(context)
                  .translate('settings_about_openlp')),
              onTap: () {
                launch('https://openlp.org/');
              },
            ),
          ],
        ),
      ),
    );
  }
}

class _InputDialog extends StatelessWidget {
  final String title;
  final String initialValue;
  final TextInputType keyboardType;
  final TextEditingController _textEditingController;

  _InputDialog(
    this.title, {
    this.initialValue,
    this.keyboardType = TextInputType.text,
  }) : _textEditingController = TextEditingController(text: initialValue ?? '');

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      title: Text(this.title),
      contentPadding: EdgeInsets.all(15),
      content: TextField(
        autofocus: true,
        controller: _textEditingController,
        keyboardType: keyboardType,
      ),
      actions: <Widget>[
        FlatButton(
          child: Text(AppLocalizations.of(context).translate('button_cancel')),
          onPressed: () {
            Navigator.pop(context, '');
          },
        ),
        FlatButton(
          child: Text(AppLocalizations.of(context).translate('button_ok')),
          onPressed: () => Navigator.pop(context, _textEditingController.text),
        ),
      ],
    );
  }
}
